/*
 * setup.s is responsible for getting the system data from the BIOS,
 * and putting them into the appropriate places in system memory.
 * both setup.s and system has been loaded by the bootblock.
 *
 * 1-Jan-96 Modified by Chris Brady for use as a boot/loader for memtest-86.
 */

#include "defs.h"
#include "config.h"

.code16
.text
.globl start
start:

# ok, the read went well so we get current cursor position and save it for
# posterity.
	movw	$INITSEG, %ax	# this is done in bootsect already, but...
	movw	%ax, %ds


/* Get memory size */
/* method E801H: */

meme801:
	movw	$0xe801, %ax
	int	$0x15
	jc	mem88

	andl	$0xffff, %edx   # clear sign extend
	movl	%edx, (0x0)     # store extended memory size

	andl	$0xffff, %ecx   # clear sign extend
	movl	%ecx, (0x4)     # and add lower memory into total size.

# Ye Olde Traditional Methode.  Returns the memory size (up to 16mb or
# 64mb, depending on the bios) in ax.
mem88:
	movb	$0x88, %ah
	int	$0x15
	movw	%ax, (0x8)

#ifdef APM_OFF
# check for APM BIOS
	movw	$0x5300, %ax    # APM BIOS installation check
	xorw	%bx, %bx
	int	$0x15
	jc	done_apm_bios   # error -> no APM BIOS

	cmpw	$0x504d, %bx    # check for "PM" signature
	jne	done_apm_bios   # no signature -> no APM BIOS

	movw	$0x5304, %ax    # Disconnect first just in case
	xorw	%bx, %bx
	int	$0x15           # ignore return code
	
	movw	$0x5301, %ax    # Real Mode connect
	xorw	%bx, %bx
	int	$0x15
	jc	done_apm_bios   # error
	
	movw	$0x5308, %ax    # Disable APM
	mov	$0xffff, %bx
	xorw	%cx, %cx
	int	$0x15
		
done_apm_bios: 
#endif

# now we want to move to protected mode ...

	cli			# no interrupts allowed #
	movb	$0x80, %al	# disable NMI for the bootup sequence
	outb	%al, $0x70

# first we move the system to its rightful place

	movw	$TESTSEG, %ax   # start of destination segment
	movw	$TSTLOAD, %bx   # start of source segment
	cld			# 'direction'=0, movs moves forward
do_move:
	movw	%ax, %es	# destination segment
	addw	$0x100, %ax
	cmpw	$INITSEG, %ax
	jz	end_move
	movw	%bx, %ds	# source segment
	addw	$0x100, %bx
	subw	%di, %di
	subw	%si, %si
	movw	$0x800, %cx
	rep
	movsw
	jmp	do_move
	
# then we load the segment descriptors

end_move:
	push	%cs
	pop	%ds
	lidt	idt_48		# load idt with 0,0
	lgdt	gdt_48		# load gdt with whatever appropriate

# that was painless, now we enable A20

	call    empty_8042
	
	movb	$0xD1, %al	# command write
	outb	%al, $0x64
	call    empty_8042	

	movb	$0xDF, %al	# A20 on
	outb	%al, $0x60
	call	empty_8042

/*
 * Note that the short jump isn't strictly needed, althought there are
 * reasons why it might be a good idea. It won't hurt in any case.
 */
	movw	$0x0001, %ax	# protected mode (PE) bit
	lmsw	%ax		# This is it#
	jmp	flush_instr
flush_instr:
	movw	$KERNEL_DS, %ax
	movw	%ax, %ds
	movw	%ax, %es
	movw	%ax, %ss
	movw	%ax, %fs
	movw	%ax, %gs

# Clear the video display
	movw	$0x0720, %ax
	movl	$0xb8000, %ebx
	movl	$0xc0000, %ecx
clrlp:
	addr32	movw	%ax, (%ebx)
	incl	%ebx
	incl	%ebx
	cmpl	%ecx, %ebx
	jnz	clrlp

#if 1
data32	ljmp	$KERNEL_CS, $LOW_TEST_ADR	# jmp offset 2000 of segment 0x10 (cs)
#else
	ljmp	$KERNEL_CS, $LOW_TEST_ADR	# jmp offset 2000 of segment 0x10 (cs)
#endif

/*
 * This routine checks that the keyboard command queue is empty
 * (after emptying the output buffers)
 *
 * No timeout is used - if this hangs there is something wrong with
 * the machine, and we probably couldn't proceed anyway.
 */
empty_8042:
	call	delay
	inb	$0x64, %al	# 8042 status port
	testb	$1, %al		# output buffer?
	jz	no_output
	call	delay
	inb	$0x60, %al	# read it
	jmp	empty_8042
	
no_output:
	testb	$2, %al		# is input buffer full?
	jnz	empty_8042	# yes - loop
	ret
#
# Delay is needed after doing i/o
#
delay:
	.word	0x00eb			# jmp $+2
	ret

gdt:
	.word	0,0,0,0		# dummy

	.word	0,0,0,0		# unused

	.word	0x7FFF		# limit 128mb
	.word	0x0000		# base address=0
	.word	0x9A00		# code read/exec
	.word	0x00C0		# granularity=4096, 386

	.word	0x7FFF		# limit 128mb
	.word	0x0000		# base address=0
	.word	0x9200		# data read/write
	.word	0x00C0		# granularity=4096, 386

idt_48:
	.word	0			# idt limit=0
	.word	0,0x3			# idt base=0L

gdt_48:
	.word	0x800		# gdt limit=2048, 256 GDT entries
	.word	512+gdt,0x9	# gdt base = 0X9xxxx

	/* Pad setup to the proper size */
	.org	(SETUPSECS*512)
