/* test.h - MemTest-86  Version 2.8
 *
 * Released under version 2 of the Gnu Public License.
 * By Chris Brady, cbrady@sgi.com
 */

#define SPINSZ		0x200000
#define DEFTESTS	7
#define MOD_SZ		20
#define BAILOUT		if (bail) goto skip_test;
#define BAILR		if (bail) return;

#define SKIP_START	0xa0000
#define SKIP_END	0x100000
#define SCREEN_ADR	0xb8000

#define TITLE_WIDTH 28
#define LINE_TIME 7
#define COL_TIME 13
#define LINE_TST 2
#define LINE_RANGE 3
#define LINE_CPU 1
#define COL_MID 30
#define LINE_INFO 7
#define COL_CACHE COL_MID+1
#define COL_PAT COL_MID+9
#define COL_TST COL_MID+21
#define COL_PASS COL_MID+25
#define COL_ERR COL_MID+32
#define LINE_HEADER 9
#define LINE_SCROLL 11
#define BAR_SIZE (78-COL_MID-9)

#define POP_W	30
#define POP_H	13
#define POP_X	(80-POP_W-2)/2
#define POP_Y	(20-POP_H)+1
#define NULL	0

/* memspeed operations */
#define MS_BCOPY	1
#define MS_COPY		2
#define MS_WRITE	3
#define MS_READ		4

#define getCx86(reg) ({ outb((reg), 0x22); inb(0x23); })
#define serial_echo_outb(v,a) outb((v),(a)+0x3f8)
#define serial_echo_inb(a)    inb((a)+0x3f8)
#define BOTH_EMPTY (UART_LSR_TEMT | UART_LSR_THRE)
/* Wait for transmitter & holding register to empty */
#define WAIT_FOR_XMITR \
 do { \
       lsr = serial_echo_inb(UART_LSR); \
 } while ((lsr & BOTH_EMPTY) != BOTH_EMPTY)

int insertaddress(unsigned long);
void printpatn(void);
void printpatn(void);
void itoa(char s[], int n); 
void reverse(char *p);
void serial_echo_init();
void serial_echo_print(char *s);
void ttyprint(int y, int x, char *s);
void ttyprintc(int y, int x, char c);
void cprint(int y,int x,char *s);
void hprint(int y,int x,unsigned long val);
void xprint(int y,int x,unsigned long val);
void aprint(int y,int x,unsigned long val);
void dprint(int y,int x,unsigned long val,int len, int right);
void movinv1(int iter, unsigned long p1, unsigned long p2);
void movinv32(int iter, unsigned long p1, unsigned long lb, unsigned long mb, int sval, int off);
void modtst(int off, int iter, unsigned long p1, unsigned long p2);
void error(long* adr,long good,long bad);
void ad_err1(long adr1,long adr2,long good,long bad);
void ad_err2(long adr,long bad);
void do_tick();
void init();
void inter();
void set_cache(int val);
void check_input();
void footer();
void scroll();
void popup();
void popdown();
void popclear();
void get_config();
void get_printmode();
void cpu_type();
void addr_tst1();
void addr_tst2();
void block_move();
void find_ticks();
void print_err();
unsigned long getval(int x, int y);
int get_key();
int cpuspeed();
long memspeed();
int check_ram();
int ascii_to_keycode(int in);
void wait_keyup();
void cacheable();
void print_hdr();
void restart();
void parity_err(unsigned long edi, unsigned long esi);
void start_config();
void clear_screen();
void xmemspeed();
int strlen();

extern long idt_descr;

#define PRINTMODE_ADDRESSES 0
#define PRINTMODE_PATTERNS  1

#define BADRAM_MAXPATNS 10
typedef unsigned long ulong;

struct pair {
       ulong adr;
       ulong mask;
};


extern __inline__ void cache_off()
{
        __asm__("push %eax\n\t"
		"movl %cr0,%eax\n\t"
                "orl $0x40000000,%eax\n\t"  /* Set CD */
                "movl %eax,%cr0\n\t"
		".byte 0x0f,0x09\n\t"	/* Invalidate and flush cache */
		"pop  %eax\n\t");
}
extern __inline__ void cache_on()
{
        __asm__("push %eax\n\t"
		"movl %cr0,%eax\n\t"
                "andl $0x9fffffff,%eax\n\t" /* Clear CD and NW */ 
                "movl %eax,%cr0\n\t"
		"pop  %eax\n\t");
}

extern __inline__ void reboot()
{
        __asm__(
		"movl %cr0,%eax\n\t"
       		"andl  $0x00000011,%eax\n\t"
       		"orl   $0x60000000,%eax\n\t"
       		"movl  %eax,%cr0\n\t"
       		"movl  %eax,%cr3\n\t"
		"movl  %cr0,%ebx\n\t"
		"andl  $0x60000000,%ebx\n\t"
		"jz    f\n\t"
		".byte 0x0f,0x09\n\t"	/* Invalidate and flush cache */
		"f: andb  $0x10,%al\n\t"
		"movl  %eax,%cr0\n\t"
		"movw $0x0010,%ax\n\t"
		"movw %ax,%ds\n\t"
		"movw %ax,%es\n\t"
		"movw %ax,%fs\n\t"
		"movw %ax,%gs\n\t"
		"movw %ax,%ss\n\t"
		"ljmp  $0xffff,$0x0000\n\t");
}

struct mmap {
	int v;
	volatile unsigned long *start;
	volatile unsigned long *end;
};

struct tseq {
	short cache;
	short pat;
	short iter;
	short ticks;
	short errors;
	char *msg;
};

struct cpu_ident {
	char type;
	char model;
	char step;
	char fill;
	long cpuid;
	long capability;
	char vend_id[12];
	unsigned char cache_info[16];
};

/* Define common variables accross relocations of memtest86 */
struct vars {
	int firsttime;
	int test;
	int pass;
	unsigned long *eadr;
	unsigned long exor;
	int msg_line;
	int ecount;
	int msegs;
	int cache_flag;
	int xtst_flag;
	int testsel;
	int scroll_start;
	int rdtsc;
	int pass_ticks;
	int total_ticks;
	int pptr;
	int tptr;
	struct mmap map[4];
	struct mmap rmap[4];
	volatile unsigned long lim_lower;
	volatile unsigned long lim_upper;
	unsigned long clks_msec;
	unsigned long starth;
	unsigned long startl;
	unsigned long snaph;
	unsigned long snapl;
	int printmode;
	int numpatn;
	struct pair patn [BADRAM_MAXPATNS];
};
